@@ -1,4 +1,4 @@
-/* $Id: gdict-defbox.c,v 1.41 2005/01/24 21:17:27 vnoel Exp $ */
+/* $Id: gdict-defbox.c,v 1.53 2005/05/25 15:01:40 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -31,6 +31,15 @@ enum {
     LAST_SIGNAL
 };
 
+typedef enum {
+	BOLD = 1,
+	ITALIC = 2,
+	UNDERLINE = 4,
+	EMPHASIS = 8,
+	GREY = 16,
+	BIG = 32
+}GDictFaces;
+
 #define LEFT_MARGIN 72.
 #define RIGHT_MARGIN 72.
 #define TOP_MARGIN 72.
@@ -56,15 +65,23 @@ void
 gdict_defbox_setup_tags (GDictDefbox *defbox)
 {
     GtkTextBuffer *buffer;
-	
+
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
-	
+
     gtk_text_buffer_create_tag (buffer, "italic",
 			        "style", PANGO_STYLE_ITALIC, NULL);
     gtk_text_buffer_create_tag (buffer, "bold",
 			        "weight", PANGO_WEIGHT_BOLD, NULL);
     gtk_text_buffer_create_tag (buffer, "underline",
 			        "underline", PANGO_UNDERLINE_SINGLE, NULL);
+		gtk_text_buffer_create_tag (buffer, "big",
+																"scale", 1.6,
+																"pixels-below-lines", 5,
+																"pixels-above-lines", 5, NULL);
+		gtk_text_buffer_create_tag (buffer, "grey",
+																"foreground", "dark gray", NULL);
+		gtk_text_buffer_create_tag (buffer, "emphasis",
+																"foreground", "dark green", NULL);
 }
 
 static void
@@ -80,6 +97,7 @@ gdict_defbox_lookup_start_cb (GtkWidget *widget, gpointer data)
   gdict_menus_set_sensitive (gdict, FALSE);
 
   gtk_window_set_title (GTK_WINDOW (gdict), _("Dictionary"));
+	gtk_widget_show (gdict->statusbar);
   gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar), 0, _("Looking up word..."));
 }
 
@@ -96,8 +114,7 @@ gdict_defbox_lookup_done_cb (GtkWidget *widget, gpointer data)
   g_free (word);
   g_free (title);
 
-  gtk_statusbar_pop (GTK_STATUSBAR (gdict->statusbar), 0);
-  gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar), 0, _("Lookup done"));
+	gtk_widget_hide (gdict->statusbar);
   gdict_menus_set_sensitive (gdict, TRUE);
 
   gdk_window_set_cursor (GTK_WIDGET(gdict)->window, NULL);
@@ -108,6 +125,7 @@ gdict_defbox_not_found_cb (GtkWidget *widget, gpointer data)
 {
   GDictWindow *gdict = data;
   gchar *word;
+	gtk_widget_show (gdict->statusbar);
   gtk_statusbar_pop (GTK_STATUSBAR (gdict->statusbar), 0);
   gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar), 0, _("No matches found"));
   word = gdict_defbox_get_word (gdict->defbox);
@@ -118,6 +136,7 @@ static void
 gdict_defbox_substr_not_found_cb (GtkWidget *widget, gpointer data) 
 {
   GDictWindow *gdict = data;
+	gtk_widget_show (gdict->statusbar);
   gtk_statusbar_pop (GTK_STATUSBAR (gdict->statusbar), 0);
   gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar), 0, _("String not found"));
 }
@@ -133,15 +152,18 @@ gdict_defbox_follow_if_link (GtkWidget *defbox, GtkTextIter *iter, GDictWindow *
     GtkTextTag *tag = tagp->data;
     str = g_object_get_data (G_OBJECT (tag), "page");
 
-    if (mousebutton == 2)
-      gdict = GDICT_WINDOW (gdict_window_new());
-    else
-      gdict_defbox_clear (GDICT_DEFBOX (defbox));
+		if (str) {
 
-    gdict_lookup_definition (gdict, str);
-    gtk_widget_grab_focus (gdict->word_entry);
-    gtk_editable_set_position (GTK_EDITABLE (gdict->word_entry), -1);
-    g_free (str);
+			if (mousebutton == 2)
+				gdict = GDICT_WINDOW (gdict_window_new());
+			else
+				gdict_defbox_clear (GDICT_DEFBOX (defbox));
+			
+			gdict_lookup_definition (gdict, str);
+			gtk_widget_grab_focus (gdict->word_entry);
+			gtk_editable_set_position (GTK_EDITABLE (gdict->word_entry), -1);
+			g_free (str);
+		}
   }
 
   if (tags) 
@@ -358,13 +380,24 @@ GtkWidget *
 gdict_defbox_new (gpointer gdict)
 {
     GDictDefbox *defbox;
+    PangoFontDescription *fontdesc;
+		PangoContext *context;
 
     defbox = GDICT_DEFBOX (g_object_new (GDICT_TYPE_DEFBOX, NULL));
     gtk_text_view_set_left_margin (GTK_TEXT_VIEW (defbox), 4);
     gtk_text_view_set_editable (GTK_TEXT_VIEW (defbox), FALSE);
 
+		context = gtk_widget_get_pango_context (GTK_WIDGET (defbox));
+		fontdesc = pango_context_get_font_description (context);
+
+		if (pango_font_description_get_family (fontdesc)) {
+			defbox->fontname = g_strdup(pango_font_description_get_family (fontdesc));
+			defbox->fontsize = pango_font_description_get_size (fontdesc);
+		}
+		
     defbox->gdict = gdict;
     defbox->context = GDICT_WINDOW(gdict)->context;
+		defbox->fontsize = 10;
     gdict_defbox_setup_tags (defbox);  
     g_signal_connect (G_OBJECT(defbox), "event-after", 
                       G_CALLBACK (gdict_defbox_event_after), gdict);
@@ -449,6 +482,47 @@ gdict_defbox_clear (GDictDefbox *defbox)
     }
 }
 
+void
+gdict_defbox_set_font (GDictDefbox *defbox, gchar *fontname)
+{
+	PangoFontDescription *fontdesc;
+
+	fontdesc = pango_font_description_from_string (fontname);
+	if (pango_font_description_get_family (fontdesc) != NULL)
+		gtk_widget_modify_font (GTK_WIDGET(defbox), fontdesc);
+
+	g_free (defbox->fontname);
+	defbox->fontname = g_strdup (fontname);
+	gdict_window_store_font (defbox->gdict);
+}
+
+void
+gdict_defbox_set_fontsize (GDictDefbox *defbox, int fontsize)
+{
+	PangoFontDescription *fontdesc;
+	PangoContext *context;
+	
+	context = gtk_widget_get_pango_context (GTK_WIDGET(defbox));
+	fontdesc = pango_context_get_font_description (context);
+	pango_font_description_set_size (fontdesc, fontsize*PANGO_SCALE);
+	gtk_widget_modify_font (GTK_WIDGET(defbox), fontdesc);
+	defbox->fontsize = fontsize;
+}	
+
+void
+gdict_defbox_zoom_in (GDictDefbox *defbox)
+{
+	defbox->fontsize = MIN (defbox->fontsize + 1, 24);
+	gdict_defbox_set_fontsize (defbox, defbox->fontsize);
+}	
+
+void
+gdict_defbox_zoom_out (GDictDefbox *defbox)
+{
+	defbox->fontsize = MAX (defbox->fontsize - 1, 6);
+	gdict_defbox_set_fontsize (defbox, defbox->fontsize);
+}	
+
 /* gdict_defbox_find
  * Finds a string of text in the current definition
  */
@@ -586,7 +660,7 @@ gdict_defbox_prepare_print (GDictDefbox *defbox)
     pc = gnome_print_job_get_context (gdict->print_job);
     g_return_if_fail(pc != NULL);
     layout = gnome_print_pango_create_layout (pc);
-    desc = pango_font_description_from_string ("Times 10");
+    desc = pango_font_description_from_string ("Serif 10");
     pango_layout_set_font_description (layout, desc);
     pango_font_description_free (desc);
     pango_layout_set_width (layout, (page_width - LEFT_MARGIN - RIGHT_MARGIN) * PANGO_SCALE);
@@ -652,7 +726,6 @@ is_xref (gchar *str, int len)
 }
 
 /* is_number
- *
  * Returns true if the substring given by str[0..len-1] is a heading number
  */
 
@@ -661,7 +734,7 @@ is_number (gchar *str, int len)
 {
     gint i;
 
-    if (str[len-1] != '.')
+    if ((str[len-1] != '.') && (str[len-1] != ':'))
         return FALSE;
         
     for (i = 0;  i < len - 1;  i++)
@@ -672,32 +745,35 @@ is_number (gchar *str, int len)
 }
 
 /* is_part
- *
  * Returns true if the substring given by str[0..len-1] is a part of speech
  */
 
 static gboolean 
 is_part (gchar *str, int len)
 {
-    gchar buf[3];
+    gchar buf[4];
 
-    if ((len < 1) || (len > 2))
+    if ((len < 1) || (len > 3))
         return FALSE;
 
-    strncpy(buf, str, 2);
+    strncpy(buf, str, 3);
     buf[len] = 0;
 
     return (strcmp(buf, "n") == 0) || \
-           (strcmp(buf, "vt") == 0) || \
-           (strcmp(buf, "vi") == 0) || \
-           (strcmp(buf, "vb") == 0) || \
-           (strcmp(buf, "aj") == 0) || \
-           (strcmp(buf, "av") == 0);
+			(strcmp(buf, "n.") == 0) || \
+			(strcmp(buf, "vt") == 0) || \
+			(strcmp(buf, "vi") == 0) || \
+			(strcmp(buf, "vb") == 0) || \
+			(strcmp(buf, "aj") == 0) || \
+			(strcmp(buf, "v") == 0) || \
+			(strcmp(buf, "adj") == 0) || \
+			(strcmp(buf, "adv") == 0) || \
+			(strcmp(buf, "av") == 0);
 }
 
 static void
 insert_text_with_tags (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *p, gint len,
-		       gboolean bold, gboolean italic, gboolean underline)
+											 int faces)
 {
 	GtkTextMark *mark;
 	GtkTextIter start;
@@ -707,12 +783,18 @@ insert_text_with_tags (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *p, gint
 	gtk_text_buffer_get_iter_at_mark (buffer, &start, mark);
 	gtk_text_buffer_delete_mark (buffer, mark);
 
-	if (bold)
+	if (faces & BIG)
+		gtk_text_buffer_apply_tag_by_name (buffer, "big", &start, iter);
+	if (faces & BOLD)
 		gtk_text_buffer_apply_tag_by_name (buffer, "bold", &start, iter);
-	if (italic)
+	if (faces & ITALIC)
 		gtk_text_buffer_apply_tag_by_name (buffer, "italic", &start, iter);
-	if (underline)
+	if (faces & UNDERLINE)
 		gtk_text_buffer_apply_tag_by_name (buffer, "underline", &start, iter);
+	if (faces & GREY)
+		gtk_text_buffer_apply_tag_by_name (buffer, "grey", &start, iter);
+	if (faces & EMPHASIS)
+		gtk_text_buffer_apply_tag_by_name (buffer, "emphasis", &start, iter);
 }
 
 static void
@@ -729,7 +811,6 @@ insert_link (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *text, gint len)
 	gtk_text_buffer_insert_with_tags (buffer, iter, text, len , tag, NULL);
 }
 
-
 /* defbox_add
  *
  * Adds a definition to the defbox, performing all necessary formatting
@@ -740,143 +821,105 @@ gdict_defbox_add (GDictDefbox *defbox, gchar *def)
 {
     GtkTextBuffer *buffer;
     GtkTextIter iter;
-    GDictWindow *gdict = defbox->gdict;
     gchar *p, *q, *text;
     gint len;
-    GdkFont *font;
-    GdkColor *color;
-    gboolean italic, bold, underline;
-    int length = 0;
+		int faces;
 
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
     gtk_text_buffer_get_end_iter (buffer, &iter);
     
-    gtk_text_buffer_insert (buffer, &iter, "\n", -1);
-         
-    font = gdict->pref->typefaces[TYPEFACE_HEADWORD].font;
-    color = gdict->pref->typefaces[TYPEFACE_HEADWORD].color;
-
-    p = defbox->def_cmd->db_full_name;
-    len = strlen(p);
-
-    gtk_text_buffer_insert (buffer, &iter, p, len);
-
     p = def;
-    len = strcspn(def, " ");
+    len = strcspn(def, " \t\n");
     
-    bold = TRUE;
-    italic = FALSE;
-    underline = TRUE; /* only for the header */
+		faces = BIG;
 
     while (*p) {
         /* Handle word token */
-        insert_text_with_tags (buffer, &iter, p, len, bold, italic, underline);
-        
-        if ((font == gdict->pref->typefaces[TYPEFACE_HEADWORD].font) || \
-            (font == gdict->pref->typefaces[TYPEFACE_NUMBER].font) || \
-            (font == gdict->pref->typefaces[TYPEFACE_PART].font) || \
-            (font == gdict->pref->typefaces[TYPEFACE_XREF].font))
-        {
-            font = gdict->pref->typefaces[TYPEFACE_BODY].font;
-            color = gdict->pref->typefaces[TYPEFACE_BODY].color;
-            bold = FALSE;
-            italic = FALSE;
-            underline = FALSE;
-        }
+        insert_text_with_tags (buffer, &iter, p, len, faces);        
+				faces = faces & ~BOLD;
+				faces = faces & ~EMPHASIS;
+
         p += len;
+
+				/* ... then handle spaces ... */
+        len = strspn(p, " \t");
         
-        /* ... then handle spaces ... */
-        len = strspn(p, " ");
-        
-        if (len > 0) {
-            insert_text_with_tags (buffer, &iter, p, len, bold, italic, underline);
-        }
-#if 0
         if (len > 0)
-            gtk_text_insert (GTK_TEXT (defbox),  font, color, NULL, p, len);
-#endif
+					insert_text_with_tags (buffer, &iter, p, len, faces);
         p += len;
 
+				/* If we just added the header word and no pronunciation, add a line break */
+				if ((faces & BIG) && (*p!='\n') && (*p!='\\'))
+					insert_text_with_tags (buffer, &iter, "\n", 1, faces);
+				faces = faces & ~BIG;
+
         /* ... handle special characters ... */
         if (*p == '\\') {
-            /*font = (font == gdict->pref->typefaces[TYPEFACE_BODY].font) ? \
-                gdict->pref->typefaces[TYPEFACE_PRONUNCIATION].font : \
-                gdict->pref->typefaces[TYPEFACE_BODY].font;*/
-            color = (color == gdict->pref->typefaces[TYPEFACE_BODY].color) ? \
-                gdict->pref->typefaces[TYPEFACE_PRONUNCIATION].color : \
-                gdict->pref->typefaces[TYPEFACE_BODY].color;
-            if (font == gdict->pref->typefaces[TYPEFACE_BODY].font) {
-                font = gdict->pref->typefaces[TYPEFACE_PRONUNCIATION].font;
-                bold = FALSE;
-                italic = TRUE;
-                underline = FALSE;
-            }
-            else {
-            	font = gdict->pref->typefaces[TYPEFACE_BODY].font;
-            	bold = FALSE;
-            	italic = FALSE;
-            	underline = FALSE;
-            }
-            ++p;
+					/* Pronunciation */
+					if (!(faces & GREY)) {
+						/* beginning the pronunciation */
+						faces |= GREY;
+						insert_text_with_tags (buffer, &iter, "| ", 2, faces);
+			    } else {
+						/* end of the pronunciation */
+						insert_text_with_tags (buffer, &iter, " |\n", 3, faces);
+						if (*(p+1) == ',')
+							p++;
+						faces = faces & ~GREY;
+					}
+					++p;
         } else if (*p == '[') {
-            font = gdict->pref->typefaces[TYPEFACE_ETYMOLOGY].font;
-            color = gdict->pref->typefaces[TYPEFACE_ETYMOLOGY].color;
-            bold = FALSE;
-            italic = TRUE;
-            underline = FALSE;
-            ++p;
+					/* Etymology or synonyms */
+					faces |= ITALIC;
+					++p;
+				} else if (*p == ']') {
+					faces = faces & ~ITALIC;
+					p++;
         } else if (*p == '{') {
-            font = gdict->pref->typefaces[TYPEFACE_EXAMPLE].font;
-            color = gdict->pref->typefaces[TYPEFACE_EXAMPLE].color;
-            bold = TRUE;
-            italic = TRUE;
-            underline = FALSE;
-			q = p;
-			while (*q != '}')
-			{
-				 length++;
-				 q++;
-                        }				
-			len = length;
-			++p;	
-			text = g_strndup (p, len - 1);
-			insert_link (buffer, &iter, text, len - 1);
-			p += length;
-			length = 0;
+					int length = 0;
+
+					/* we must take care in this function
+					 * not to loop after the \0 of the string */
+					/* search the last '{' of a sequence of '{' */
+					while (*(p+1) == '{')
+						p++;
+					
+					q = p;
+					
+					/* search the first '}' after the link text */
+					while (*q != '\0' && *q != '}') {
+						length++;
+						q++;
+					}
+					
+					len = length;
+					++p; /* go to the char behind the last '{' */
+					text = g_strndup (p, len - 1);
+					insert_link (buffer, &iter, text, len - 1);
+					p += length; /* on first '}' */
+					
+					/* step through the remaining '}' */
+					while (*p == '}')
+						p++;
+					length = 0;
+				} else if (*p == '}' ) {
+					/* unbalanced {} pairs, just go behind the closing... */
+					while (*p == '}')
+						p++;
         } else if (*p == ']' ) {
-            font = gdict->pref->typefaces[TYPEFACE_BODY].font;
-            color = gdict->pref->typefaces[TYPEFACE_BODY].color;
-            bold = FALSE;
-            italic = FALSE;
-            underline = FALSE;
-            ++p;
+					++p;
         }
         
         len = strcspn (p, " \\[]{}");
 
-        if (font == gdict->pref->typefaces[TYPEFACE_BODY].font) {
-            if (is_xref(p, len)) {
-                font = gdict->pref->typefaces[TYPEFACE_XREF].font;
-                color = gdict->pref->typefaces[TYPEFACE_XREF].color;
-                bold = FALSE;
-                italic = FALSE;
-                underline = FALSE;
-            } else if (is_number(p, len)) {
-                font = gdict->pref->typefaces[TYPEFACE_NUMBER].font;
-                color = gdict->pref->typefaces[TYPEFACE_NUMBER].color;
-                bold = TRUE;
-                italic = FALSE;
-                underline = FALSE;
-            } else if (is_part(p, len)) {
-                font = gdict->pref->typefaces[TYPEFACE_PART].font;
-                color = gdict->pref->typefaces[TYPEFACE_PART].color;
-                bold = FALSE;
-                italic = FALSE;
-                underline = FALSE;
-            }
-        }
+				if (is_number(p, len)) {
+					faces = faces | BOLD;
+					if (*p=='1')
+						insert_text_with_tags (buffer, &iter, "\n    ", 5, faces);
+				}
+				else if (is_part(p, len))
+					faces = faces | (BOLD | EMPHASIS);
     }    
-   
 }
 
 /* gdict_defbox_error_cb
