@@ -19,23 +19,13 @@
 
     ---------------------------------------------------------------------- */
 
-#include <config.h>
-#include <sys/stat.h>
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include "logview.h"
 #include "logrtns.h"
 #include "log_repaint.h"
 #include "misc.h"
 
-/* 
- * -------------------
- * Global variables 
- * -------------------
- */
-
-extern UserPrefsStruct *user_prefs;
-extern gchar *file_to_open;
-
 /*
  * -------------------
  * Module variables 
@@ -54,6 +44,9 @@ enum {
    MESSAGE
 };
 
+static void DrawLogLines (LogviewWindow *window, Log *current_log);
+static void UpdateStatusArea (LogviewWindow *window);
+
 /**
  * Recursively called until the row specified by orig is found.
  *
@@ -196,13 +189,8 @@ handle_row_activation_cb (GtkTreeView *treeview, GtkTreePath *path,
     gtk_tree_path_free (root_tree);
 
     window->curlog->current_line_no = row;
-
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM
-				    (gtk_ui_manager_get_widget (window->ui_manager, "/LogviewMenu/ViewMenu/ShowDetails")),
-				    TRUE);
 }
 
-
 /* ----------------------------------------------------------------------
    NAME:        handle_selection_changed_cb
    DESCRIPTION: User clicked on main window
@@ -211,29 +199,32 @@ handle_row_activation_cb (GtkTreeView *treeview, GtkTreePath *path,
 void
 handle_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 {
-    GtkTreeIter iter;
     GtkTreeModel *model;
-    GtkTreePath *path, *root_tree;
-    gint row = 0;
+    GList *selected_paths, *i;
+    int selected_first = -1, selected_last = -1;
     LogviewWindow *window = data;
+    GtkTreePath *selected_path;
 
-    if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+    selected_paths = gtk_tree_selection_get_selected_rows (selection, &model);
 
-        path = gtk_tree_model_get_path (model, &iter);
-        root_tree = gtk_tree_path_new_root ();
-
-        window->curlog->current_path = gtk_tree_path_copy (path);
-        iterate_thru_children (GTK_TREE_VIEW (window->view), model, root_tree, path, &row, 0);
-
-        gtk_tree_path_free (root_tree);
-        gtk_tree_path_free (path);
-
-        window->curlog->current_line_no = row;
-
-	if (window->zoom_visible)
-		log_redrawdetail (window);
+    if (selected_paths) {
+	    for (i = selected_paths; i != NULL; i = g_list_next (i)) {		    
+		    GtkTreePath *root_tree = gtk_tree_path_new_root ();
+		    int row = 0;
+		    selected_path = i->data;
+		    iterate_thru_children (GTK_TREE_VIEW (window->view), model, root_tree, selected_path, &row, 0);		    
+		    if (selected_last == -1 || row > selected_last)
+			    selected_last = row;
+		    if (selected_first == -1 || row < selected_first)
+			    selected_first = row;
+	    }
+	    window->curlog->current_line_no = selected_last;
+	    window->curlog->current_path = gtk_tree_path_copy (selected_path);
+	    window->curlog->selected_line_first = selected_first;
+	    window->curlog->selected_line_last = selected_last;
+	    g_list_foreach (selected_paths, (GFunc) gtk_tree_path_free, NULL);
+	    g_list_free (selected_paths);
     }
-
 }
 
 /* ----------------------------------------------------------------------
@@ -252,14 +243,21 @@ log_repaint (LogviewWindow *window)
    tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->view));
    gtk_tree_store_clear (GTK_TREE_STORE (tree_model));
 
-   log_redrawdetail (window); 
    UpdateStatusArea (window);	   
 
    /* Check that there is at least one log */
    if (window->curlog == NULL) {
 	   return FALSE;
    }
-   
+	 
+	 if (window->curlog->monitored) {
+		 gtk_widget_hide (window->log_scrolled_window);
+		 gtk_widget_show (window->mon_scrolled_window);
+	 } else {
+		 gtk_widget_hide (window->mon_scrolled_window);
+		 gtk_widget_show (window->log_scrolled_window);
+	 }
+
    /* Draw the tree view */ 
    DrawLogLines (window, window->curlog); 
 
@@ -295,7 +293,7 @@ UpdateStatusArea (LogviewWindow *window)
        } else {
            utf8 = LocaleToUTF8 (status_text);
        }
-       statusbar_text = g_strconcat (_("Last Modified: "), utf8, NULL);
+       statusbar_text = g_strconcat (_("Last Modified:"), utf8, NULL);
        gtk_statusbar_pop (GTK_STATUSBAR(window->statusbar), 0);
        gtk_statusbar_push (GTK_STATUSBAR(window->statusbar), 0, statusbar_text);
        
@@ -444,15 +442,3 @@ DrawLogLines (LogviewWindow *window, Log *current_log)
    g_free (utf8);
    
 }
-
-/* ----------------------------------------------------------------------
-   NAME:        log_redrawdetail
-   DESCRIPTION: Redraw area were the detailed information is drawn.
-   ---------------------------------------------------------------------- */
-
-void
-log_redrawdetail (LogviewWindow *window)
-{
-  if (window->zoom_visible)
-    repaint_zoom (window);
-}
